from ..utils import *
from ..Agent import Agent as BaseAgent

class ProductManager(BaseAgent):

    def __init__(self, role= '', 
                 competencies='', 
                 capabilities= '',
                 tone= '',
                 lang= '',
                 extra_instructions= '',
                 openai_api_key: str=AGENTS_OPENAI_API_KEY, 
                 model: str=AGENTS_MODEL,
                 temperature: float=AGENTS_MODEL_TEMP) -> None:

        super().__init__(role, 
                 competencies, 
                 capabilities,
                 tone,
                 lang,
                 extra_instructions,
                 openai_api_key, 
                 model,
                 temperature)
        
        self.role = 'Product Manager'
        self.competencies = """
        -BS in Computer Science or equivalent industry experience
        -2+ years of product management or related industry experience
        -1+ years of experience working on a B2B SaaS product
        -High level of understanding of web development practices
        -Strong understanding of digital analytics and conversion rate optimization processes
        -Strong ability to refine and priorities product backlog in fast-moving environment
        -Strong understanding of the Digital ecosystem, website design, content types, mobile and tablet applications, web technologies and digital marketing best practices.
        -Proficient at writing user stories and defining acceptance criteria
        -Experience going through a full product lifecycle, integrating customer feedback into product requirements.
        -Excellent analytical skills, with the ability to interrogate project metrics to identify areas for continuous improvement
        -Excellent written and verbal communication skills with business and technicalstakeholders at all levels
        """
        self.capabilities = """
        -Owning the product vision and key objectives ensuring this aligns to the overall Business goals.
        -Delivering and maintaining the roadmap and product strategy
        -Producing product performance reports
        -Partnering with UX team to build delightful features with solid designs
        -Transforming ideas and feedback into a product vision and roadmap
        -Understanding with customer problems in order to collect user needs, requirements, and pain points
        -Utilizing performance metrics and data analysis to refine and improve product impact
        -Working closely with the Technical team to execute and deliver roadmap milestones.
        -Understand, communicate and drive optimization of all leading metrics and KPIs 
        """

    def general_answer(self, question: str, context: str=None):
        return super().general_answer(question=question, context=context)


    def write_user_story(self, context: str):
        """
        Write the best user story based on the given context.

        Args:
            context (str): The context for writing the user story.

        Returns:
            str: The detailed user story including acceptance criteria.
        """

        # Define the task and question for the user story
        task = """Considering your competencies and utilizing your capabilities, given the above CONTEXT, your task is to write the best user story."""
        question = """Write the detailed user story including its acceptance criteria in the following format:
        Background: [one sentence]
        Scenario: [the user story]
        Acceptance Criteria: [bullet points]"""

        # Call the openai_answer function to generate the user story
        response = openai_answer(role=self.role,
                                competencies=self.competencies,
                                capabilities=self.capabilities,
                                task=task,
                                question=question,
                                context=context,
                                lang=self.lang,
                                tone=self.tone,
                                extra_instructions=self.extra_instructions,
                                openai_api_key=self.openai_api_key,
                                model=self.model)
        return response
    
    def write_release_notes(self, context: str):
        """
        Write the release notes based on the given context.

        Args:
            context (str): The context for writing the release notes.

        Returns:
            str: The detailed release notes.
        """

        # Define the task and question for the release notes
        task = """Considering your competencies and utilizing your capabilities, given the above CONTEXT, your task is to write the release notes."""
        question = """Write the detailed release notes including its acceptance criteria in the following format:
        Overview
        [brief overview in less than 5 sentences]
        
        New Features 
        [review of the new features in bullet points]
        
        Features Enhancements
        [review of the enhancements in bullet points]
        
        Fixed Issues
        [review of the bug fixes and fixed issues in bullet points]"""

        # Call the openai_answer function to generate the user story
        response = openai_answer(role=self.role,
                                competencies=self.competencies,
                                capabilities=self.capabilities,
                                task=task,
                                question=question,
                                context=context,
                                lang=self.lang,
                                tone=self.tone,
                                extra_instructions=self.extra_instructions,
                                openai_api_key=self.openai_api_key,
                                model=self.model)
        return response


import os

# OpenAI
AGENTS_OPENAI_API_KEY = os.environ.get('OPENAI_API_KEY')
AGENTS_MODEL = os.environ.get('AGENTS_MODEL') if os.environ.get('AGENTS_MODEL') is not None else "gpt-3.5-turbo"
AGENTS_MODEL_TEMP = os.environ.get('AGENTS_MODEL_TEMP') if os.environ.get('AGENTS_MODEL_TEMP') is not None else 0



from ..utils import *
from ..Agent import Agent as BaseAgent

class FullStackDeveloper(BaseAgent):

    def __init__(self, role= '', 
                 competencies='', 
                 capabilities= '',
                 tone= '',
                 lang= '',
                 extra_instructions= '',
                 openai_api_key: str=AGENTS_OPENAI_API_KEY, 
                 model: str=AGENTS_MODEL,
                 temperature: float=AGENTS_MODEL_TEMP) -> None:

        super().__init__(role, 
                 competencies, 
                 capabilities,
                 tone,
                 lang,
                 extra_instructions,
                 openai_api_key, 
                 model,
                 temperature)
        
        self.role = 'Full Stack Software Engineer'
        self.competencies = """
        - Bachelor’s degree with a concentration in Computer Science, Computer Engineering or related subject
        - 5+ years of experience in Back-end engineering.
        - 5+ years of experience in developing highly interactive web applications.
        - Strong knowledge and first-hand experience frontend and backend stacks, languages and frameworks.
        - Strong knowledge and first-hand experience in web applications; modular and reusable.
        - Experience in Microservices architecture, RabbitMQ, Node.js, Redis, Memcache or Varnish.
        - Knowledge in Object-Oriented Design Patterns.
        - Knowledge in Enterprise Integration Patterns.
        - Strong communication skills and ability to work in a dynamic environment.
        """
        self.capabilities = """
        - Build robust and scalable software applications, APIs, services and middle-ware components as well as modifications of existing software.
        - Design and create services and define system architecture for your projects, and contribute and provide feedback to other team members.
        - Enhance code quality through writing unit tests, automation and performing code reviews.
        - Brainstorm for ideas to technologies, platform and products and see your ideas grow and flourish.
        - Work with the product and design teams to understand end-user requirements, formulate use cases, and then translate that into a pragmatic and effective technical solution.
        - Solve challenging technical problems and successfully deliver results on schedule.
        - Developing new and maintaining existing user-facing features
        - Write client-side and server-side code for web-based applications, create fast, easy-to-use, high volume production applications.
        - Ensuring the technical feasibility of UI/UX designs.
        - Contribute to presentation layer standards and practices.
        - Contribute to testing and implementation approach for presentation layer.
        - Prototype future interfaces.
        - Collaborating closely with the product team and other team members and stakeholders .
        """

    def general_answer(self, question: str, context: str=None):
        return super().general_answer(question=question, context=context)

    def write_code_documentation(self, context: str): 

        task = """Considering your competencies and utilizing your capabilities, given the above CONTEXT, your task is to write the necessary code documentation"""
        question = """Add the necessary docstrings in google style and the important inline comments to the code presented in the CONTEXT WITHOUT changing the code itself"""

        response = openai_answer(role=self.role,
                                 competencies=self.competencies,
                                 capabilities=self.capabilities,
                                 task=task,
                                 question=question,
                                 context=context,
                                 lang=self.lang,
                                 tone=self.tone,
                                 extra_instructions=self.extra_instructions,
                                 openai_api_key=self.openai_api_key,
                                 model=self.model)
        return response



from .utils import *
from .config import *

class Agent:

    def __init__(self, role= '', 
                 competencies='', 
                 capabilities= '',
                 tone= '',
                 lang= 'same',
                 extra_instructions= '',
                 openai_api_key: str=AGENTS_OPENAI_API_KEY, 
                 model: str=AGENTS_MODEL,
                 temperature: float=AGENTS_MODEL_TEMP) -> None:
        
        self.role = role
        self.competencies = competencies
        self.capabilities = capabilities
        self.tone = tone
        self.lang = lang
        self.extra_instructions = extra_instructions
        self.openai_api_key = openai_api_key
        self.model = model
        self.temperature = temperature

    def general_answer(self, question: str, context: str=None):

        task = """Given the above CONTEXT, your task is to answer the following question: """

        response = openai_answer(role=self.role,
                                 competencies=self.competencies,
                                 capabilities=self.capabilities,
                                 task=task,
                                 question=question,
                                 context=context,
                                 lang=self.lang,
                                 tone=self.tone,
                                 extra_instructions=self.extra_instructions,
                                 openai_api_key=self.openai_api_key,
                                 model=self.model)
        return response


from ..utils import *
from ..Agent import Agent as BaseAgent

class ContentCreator(BaseAgent):

    def __init__(self, role= '', 
                 competencies='', 
                 capabilities= '',
                 tone= '',
                 lang= '',
                 extra_instructions= '',
                 openai_api_key: str=AGENTS_OPENAI_API_KEY, 
                 model: str=AGENTS_MODEL,
                 temperature: float=AGENTS_MODEL_TEMP) -> None:

        super().__init__(role, 
                 competencies, 
                 capabilities,
                 tone,
                 lang,
                 extra_instructions,
                 openai_api_key, 
                 model,
                 temperature)
        
        self.role = 'Content Creator'
        self.competencies = """
        Expert in personal and business branding tactics
        Outstanding copywriting, social media, or video production skills
        Strong knowledge of SEO best practices
        Significant experience creating high-quality marketing content
        Capable of managing multiple competing priorities
        Comprehensive understanding of impactful marketing tactics
        Identify customers’ needs and recommend new topics

        """
        self.capabilities = """
        
        Create and publish engaging content for various platforms such as website, social media, blog, etc.
        Use SEO best practices to drive traffic to digital channels
        Collaborate across teams to determine and solve campaign objectives
        Monitor digital engagement metrics
        Promote offerings to reach new audiences
        Research market trends and developments relevant to campaign subject matter

        """

    def general_answer(self, question: str, context: str=None):
        return super().general_answer(question=question, context=context)

    def write_linkedin_post(self, context: str): 

        task = """Considering your competencies and utilizing your capabilities, given the above CONTEXT, your task is to write a social media post that suits LinkedIn."""
        question = """Write a compelling LinkedIn post rich in details that considers the best practices and promotes users to engage and interact for the post to go viral for the audience most relevant to the CONTEXT."""

        response = openai_answer(role=self.role,
                                 competencies=self.competencies,
                                 capabilities=self.capabilities,
                                 task=task,
                                 question=question,
                                 context=context,
                                 lang=self.lang,
                                 tone=self.tone,
                                 extra_instructions=self.extra_instructions,
                                 openai_api_key=self.openai_api_key,
                                 model=self.model)
        return response
    
    def write_facebook_post(self, context: str): 

        task = """Considering your competencies and utilizing your capabilities, given the above CONTEXT, your task is to write a social media post that suits Facebook."""
        question = """Write a compelling rich in details Facebook post that considers the best practices and promotes users to engage and interact for the post to go viral for the audience most relevant to the CONTEXT."""

        response = openai_answer(role=self.role,
                                 competencies=self.competencies,
                                 capabilities=self.capabilities,
                                 task=task,
                                 question=question,
                                 context=context,
                                 lang=self.lang,
                                 tone=self.tone,
                                 extra_instructions=self.extra_instructions,
                                 openai_api_key=self.openai_api_key,
                                 model=self.model)
        return response
    
    def write_blog_article(self, context: str): 

        task = """Considering your competencies and utilizing your capabilities, given the above CONTEXT, your task is to write a blog article."""
        question = """Write a compelling long blog article rich in details that considers the best SEO practices using the appropriate keywords for the audience most relevant to the CONTEXT."""

        response = openai_answer(role=self.role,
                                 competencies=self.competencies,
                                 capabilities=self.capabilities,
                                 task=task,
                                 question=question,
                                 context=context,
                                 lang=self.lang,
                                 tone=self.tone,
                                 extra_instructions=self.extra_instructions,
                                 openai_api_key=self.openai_api_key,
                                 model=self.model)
        return response